import inspect
import tempfile
import traceback
from typing import List

import dataframe_image as dtf

from pandas import DataFrame


def get_current_file_name() -> str:
    stack = inspect.stack()
    caller_frame = stack[len(stack) - 1]
    return caller_frame.filename


def df_to_png(df: DataFrame) -> str:
    with tempfile.NamedTemporaryFile(delete=False, suffix=".png") as temp_file:
        dtf.export(
            df,
            filename=temp_file,
            table_conversion='matplotlib'
        )
        return temp_file.name


def split_telegram_message(message: str) -> List[str]:
    return [message[i: i + 4096] for i in range(0, len(message), 4096)]


def get_exception_text(e: Exception) -> str:
    return "".join(traceback.format_exception(type(e), e, e.__traceback__))



import tempfile
import traceback
import aiohttp

from typing import Union
from pandas import DataFrame
from tenacity import (RetryError, retry, retry_if_exception_type,
                      stop_after_attempt, wait_fixed)

from .abs_channel import AbstractChannel
from .exceptions import TelegramException
from .helpers import df_to_png, split_telegram_message, get_exception_text


class AsyncTelegramChannel(AbstractChannel):
    """
    Канал для отправки статистики через Telegram.
    """
    MAX_TIMEOUT = 60

    def __init__(self, bot_token: str, chat_id: Union[str, int]) -> None:
        self.bot_token = bot_token
        self.chat_id = chat_id
        self.timeout = aiohttp.ClientTimeout(
            total=10,
            connect=3
        )

    async def send_message(self, message: str) -> None:
        if not message:
            return
        try:
            message = self._prepare_message(message)
            for sub_message in split_telegram_message(message):
                await self._send_message(
                    text=sub_message
                )
        except RetryError:
            traceback.print_exc()

    async def send_as_png(self, df: DataFrame, caption: str = "") -> None:
        f_name = df_to_png(df)
        try:
            caption = self._prepare_message(caption)
            await self._send_photo(f_name, caption)
        except RetryError:
            traceback.print_exc()

    async def send_as_xmlx(self, df: DataFrame, caption: str = "") -> None:
        try:
            with tempfile.NamedTemporaryFile(delete=True, suffix=".xlsx") as temp_file:
                df.to_excel(temp_file.name, index=False)
                caption = self._prepare_message(caption)
                await self._send_document(
                    document_path=temp_file.name,
                    caption=caption
                )
        except TelegramException:
            traceback.print_exc()

    async def send_exception(self, e: Exception, caption: str = "") -> None:
        exception_text = get_exception_text(e)
        if caption:
            full_text = "📝 " + caption + "\n" + exception_text
        else:
            full_text = exception_text
        await self.send_message(full_text)

    @retry(
        wait=wait_fixed(5),
        stop=stop_after_attempt(3),
        retry=retry_if_exception_type(TelegramException),
        reraise=True
    )
    async def _send_message(
            self,
            text: str
    ) -> None:
        conn = aiohttp.TCPConnector(ssl=False)
        async with aiohttp.ClientSession(connector=conn, timeout=self.timeout) as session:
            url = f'https://api.telegram.org/bot{self.bot_token}/sendMessage'
            data = {
                'chat_id': self.chat_id,
                'text': text
            }
            async with session.post(url, json=data) as response:
                if response.status != 200:
                    raise TelegramException(response.text)

    @retry(
        wait=wait_fixed(5),
        stop=stop_after_attempt(3),
        retry=retry_if_exception_type(TelegramException),
        reraise=True
    )
    async def _send_photo(
            self,
            photo_path: str,
            caption: str,
    ) -> None:
        conn = aiohttp.TCPConnector(ssl=False)
        with open(photo_path, 'rb') as file:
            async with aiohttp.ClientSession(connector=conn, timeout=self.timeout) as session:
                url = f'https://api.telegram.org/bot{self.bot_token}/sendPhoto'
                data = aiohttp.FormData()
                data.add_field('chat_id', str(self.chat_id))
                data.add_field('caption', caption)
                data.add_field('photo', file)

                async with session.post(url, data=data) as response:
                    if response.status != 200:
                        raise TelegramException(response.text)

    @retry(
        wait=wait_fixed(5),
        stop=stop_after_attempt(3),
        retry=retry_if_exception_type(TelegramException),
        reraise=True
    )
    async def _send_document(
            self,
            document_path: str,
            caption: str
    ) -> None:
        conn = aiohttp.TCPConnector(ssl=False)
        with open(document_path, 'rb') as file:
            async with aiohttp.ClientSession(connector=conn, timeout=self.timeout) as session:
                url = f'https://api.telegram.org/bot{self.bot_token}/sendDocument'
                data = aiohttp.FormData()
                data.add_field('chat_id', str(self.chat_id))
                data.add_field('caption', caption)
                data.add_field('document', file, filename=file.name)

                async with session.post(url, data=data) as response:
                    if response.status != 200:
                        raise TelegramException(response.text)



from abc import abstractmethod, ABC

from pandas import DataFrame

from fps_channels.helpers import get_current_file_name


class AbstractChannel(ABC):
    """
     Канал для отправки статистики.
     """
    SHOW_FILENAME = False
    HEADER = ""

    @abstractmethod
    def send_message(self, message: str):
        pass

    @abstractmethod
    def send_as_xmlx(self, stat: DataFrame, caption: str) -> None:
        ...

    @abstractmethod
    def send_as_png(self, stat: DataFrame, caption: str) -> None:
        ...

    @abstractmethod
    def send_exception(self, e: Exception) -> None:
        ...

    def _prepare_message(self, message: str) -> str:
        if self.SHOW_FILENAME:
            file_name = get_current_file_name()
            message = f"📂 file {file_name} \n{message}"

        if self.HEADER:
            message = self.HEADER + "\n" + message

        return message



import os

from setuptools import setup, find_packages


def read(file_name):
    return open(os.path.join(os.path.dirname(__file__), file_name)).read()


setup(
    name="fps_channels",
    version="1.3",
    packages=find_packages(),
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    install_requires=[
        "pandas~=2.1.3",
        "dataframe-image~=0.2.3",
        "tenacity~=8.2.3",
        "lxml~=4.9.2",
        "openpyxl~=3.1.1",
        "aiohttp~=3.9.5",
        "requests~=2.32.3"
    ]
)
